@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2007 SWsoft. All rights reserved.
+ *  Copyright (C) 2000-2008, Parallels, Inc. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,12 +20,12 @@
 
 struct iptables_s {
 	char *name;
-	unsigned long id;
+	unsigned long long id;
 	unsigned long long mask;
 };
 
 struct iptables_s *find_ipt(const char *name);
-void ipt_mask2str(unsigned long mask, char *buf, int size);
-unsigned long long get_ipt_mask(unsigned long mask);
+void ipt_mask2str(unsigned long long mask, char *buf, int size);
+unsigned long long get_ipt_mask(unsigned long long mask);
 
 #endif //_IPTABLES_H_
