@@ -24,19 +24,26 @@
 #include <unistd.h>
 #include <linux/capability.h>
 #include <string.h>
+#include <linux/vzcalluser.h>
 
 #include "cap.h"
+#include "res.h"
 #include "vzerror.h"
 #include "logger.h"
+#include "util.h"
 
-#ifndef	CAP_SETVEID
-#define	CAP_SETVEID	29
+#ifndef	CAP_AUDIT_WRITE
+#define	CAP_AUDIT_WRITE	29
 #endif
 #ifndef	CAP_VE_ADMIN
 #define	CAP_VE_ADMIN	30
 #endif
+#ifndef	CAP_SETFCAP
+#define	CAP_SETFCAP	31
+#endif
 
-#define CAPDEFAULTMASK	(CAP_TO_MASK(CAP_CHOWN)		| \
+#define CAPDEFAULTMASK_OLD				  \
+	CAP_TO_MASK(CAP_CHOWN)				| \
 	CAP_TO_MASK(CAP_DAC_OVERRIDE)			| \
 	CAP_TO_MASK(CAP_DAC_READ_SEARCH)		| \
 	CAP_TO_MASK(CAP_FOWNER)				| \
@@ -59,7 +66,12 @@
 	CAP_TO_MASK(CAP_MKNOD)				| \
 	CAP_TO_MASK(CAP_LEASE)				| \
 	CAP_TO_MASK(CAP_VE_ADMIN)			| \
-	CAP_TO_MASK(CAP_SETVEID))
+	CAP_TO_MASK(CAP_AUDIT_WRITE)
+
+#define CAPDEFAULTMASK					  \
+	CAPDEFAULTMASK_OLD				| \
+	CAP_TO_MASK(CAP_SETPCAP)			| \
+	CAP_TO_MASK(CAP_SETFCAP)
 
 static char *cap_names[] = {
 "CHOWN",		/*	0	*/
@@ -91,8 +103,9 @@ static char *cap_names[] = {
 "SYS_TTY_CONFIG",	/*	26	*/
 "MKNOD",		/*	27	*/
 "LEASE",		/*	28	*/
-"SETVEID",		/*	29	*/
+"AUDIT_WRITE",		/*	29	*/
 "VE_ADMIN",		/*	30	*/
+"SETFCAP",		/*	31	*/
 
 "FS_MASK"		/*	0x1f	*/
 };
@@ -116,7 +129,7 @@ int get_cap_mask(char *name, unsigned long *mask)
 {
 	unsigned int i;
 
-	for (i = 0; i < sizeof(cap_names) / sizeof(*cap_names); i++) {
+	for (i = 0; i < ARRAY_SIZE(cap_names); i++) {
 		if (!strcasecmp(name, cap_names[i])) {
 			cap_raise(*mask, i);
 			return 0;
@@ -141,7 +154,7 @@ void build_cap_str(cap_param *new, cap_param *old, char *buf, int len)
 	sp = buf;
 	ep = buf + len;
 	sp += sprintf(sp, "\"");
-	for (i = 0; i < sizeof(cap_names) / sizeof(*cap_names); i++) {
+	for (i = 0; i < ARRAY_SIZE(cap_names); i++) {
 		int op = 0;
 
 		if (CAP_TO_MASK(i) & new->on)
@@ -178,28 +191,30 @@ static int set_cap(envid_t veid, cap_t mask, int pid)
 	data.permitted = mask;
 	data.inheritable = mask;
 
-	if (capset(&header, &data) < 0)	{
-		logger(-1, errno, "Unable to set capability");
-		return VZ_SET_CAP;
-	}
-	return 0;
+	return capset(&header, &data);
 }
 
-static inline cap_t make_cap_mask(cap_t on, cap_t off)
+static inline cap_t make_cap_mask(cap_t def, cap_t on, cap_t off)
 {
-	return (CAPDEFAULTMASK | on) & ~off;
+	return (def | on) & ~off;
 }
 
-/** Apply capability mask to CT.
- * @param veid		CT ID.
- * @param cap		capability mask.
- * @return		0 on success.
- */
-int vps_set_cap(envid_t veid, cap_param *cap)
+int vps_set_cap(envid_t veid, struct env_param *env, cap_param *cap)
 {
 	cap_t mask;
 
-	mask = make_cap_mask(cap->on, cap->off);
+	if ((env->features_known & env->features_mask) & VE_FEATURE_BRIDGE)
+		cap_raise(cap->on, CAP_NET_ADMIN);
 
-	return set_cap(veid, mask, 0);
+	mask = make_cap_mask(CAPDEFAULTMASK, cap->on, cap->off);
+	if (set_cap(veid, mask, 0) == 0)
+		return 0;
+
+	/* Probably old kernel -- retry with old default mask */
+	mask = make_cap_mask(CAPDEFAULTMASK_OLD, cap->on, cap->off);
+	if (set_cap(veid, mask, 0) == 0)
+		return 0;
+
+	logger(-1, errno, "Unable to set capability");
+	return VZ_SET_CAP;
 }
