@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2007 SWsoft. All rights reserved.
+ *  Copyright (C) 2000-2009, Parallels, Inc. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -23,6 +23,7 @@
 #include <errno.h>
 
 #include "types.h"
+#include "bitmap.h"
 #include "cpu.h"
 #include "env.h"
 #include "vzerror.h"
@@ -59,6 +60,29 @@ static inline int fairsched_vcpus(unsigned int id, unsigned vcpus)
 	return ret;
 }
 
+#if defined(__i386__) || defined(__x86_64__)
+static inline int fairsched_cpumask(unsigned int id,
+		unsigned int masksize, unsigned long *mask)
+{
+	int ret;
+
+	ret = syscall(__NR_fairsched_cpumask, id, masksize, mask);
+	if (ret && errno == ENOSYS)
+		ret = 0;
+	return ret;
+}
+#else
+/*
+ * fairsched_cpumask is available only in vz kernels based on linux 2.6.32
+ * or later which do not support platforms different from x86.
+ */
+static inline int fairsched_cpumask(unsigned int id,
+		unsigned int masksize, unsigned long *mask)
+{
+	return ENOTSUP;
+}
+#endif
+
 static int set_cpulimit(envid_t veid, unsigned int cpulimit)
 {
 	unsigned cpulim1024 = (float)cpulimit * 1024 / 100;
@@ -98,21 +122,34 @@ static int set_cpuunits(envid_t veid, unsigned int cpuunits)
 	return ret;
 }
 
-/** Change number of CPUs available in the running VE.
- *
- * @param veid		VE id
- * @param vcpu		number of cpu
- */
-int env_set_vcpus(envid_t veid, unsigned int vcpus)
+static int set_cpumask(envid_t veid, cpumask_t *mask)
 {
-	int ret;
+	static char maskstr[CPUMASK_NBITS * 2];
 
+	bitmap_snprintf(maskstr, CPUMASK_NBITS * 2,
+			cpumask_bits(mask), CPUMASK_NBITS);
+	logger(0, 0, "Setting CPU mask: %s", maskstr);
+	if (fairsched_cpumask(veid, sizeof(cpumask_t), cpumask_bits(mask))) {
+		logger(-1, errno, "fairsched_cpumask");
+		return VZ_SETFSHD_ERROR;
+	}
+	return 0;
+}
+
+/** Change number of CPUs available in the running CT.
+ *
+ * @param veid		CT ID
+ * @param vcpu		number of CPUs
+ */
+static int env_set_vcpus(envid_t veid, unsigned int vcpus)
+{
 	logger(0, 0, "Setting CPUs: %d", vcpus);
-	ret = fairsched_vcpus(veid, vcpus);
-	if (ret)
+	if (fairsched_vcpus(veid, vcpus) != 0) {
 		logger(-1, errno, "Unable to set cpus");
+		return VZ_SETFSHD_ERROR;
+	}
 
-	return ret;
+	return 0;
 }
 
 
@@ -128,10 +165,10 @@ int hn_set_cpu(cpu_param *cpu)
 	return set_cpuunits(2147483647, *cpu->units);
 }
 
-/**  Apply cpu parameters on running VE.
+/**  Apply cpu parameters on running CT.
  *
- * @param h		VE handler.
- * @param veid		VE id.
+ * @param h		CT handler.
+ * @param veid		CT ID.
  * @param cpu		cpu parameters.
  * @return		0 on success.
  */
@@ -142,13 +179,14 @@ int vps_set_cpu(vps_handler *h, envid_t veid, cpu_param *cpu)
 	if (cpu->limit == NULL &&
 		cpu->units == NULL &&
 		cpu->weight == NULL &&
-		cpu->vcpus == NULL)
+		cpu->vcpus == NULL &&
+		cpu->mask == NULL)
 	{
 		return 0;
 	}
 	if (!vps_is_run(h, veid)) {
 		logger(-1, 0, "Unable to apply CPU parameters: "
-			"VE is not running");
+			"container is not running");
 		return VZ_VE_NOT_RUNNING;
 	}
 	if (cpu->limit != NULL) {
@@ -161,8 +199,9 @@ int vps_set_cpu(vps_handler *h, envid_t veid, cpu_param *cpu)
 	if (cpu->vcpus != NULL) {
 		ret = env_set_vcpus(veid, *cpu->vcpus);
 	}
+	if (cpu->mask != NULL) {
+		ret = set_cpumask(veid, cpu->mask);
+	}
 
 	return ret;
 }
-
-
