@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2008, Parallels, Inc. All rights reserved.
+ *  Copyright (C) 2000-2009, Parallels, Inc. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 #include <unistd.h>
 #include <fcntl.h>
 #include <string.h>
+#include <limits.h>
 
 #include "types.h"
 #include "ub.h"
@@ -31,7 +32,7 @@
 #include "vzsyscalls.h"
 
 static inline int setublimit(uid_t uid, unsigned long resource,
-	unsigned long *rlim)
+	const unsigned long *rlim)
 {
 	return syscall(__NR_setublimit, uid, resource, rlim);
 }
@@ -61,6 +62,7 @@ static struct ubname2id {
 	{"DCACHESIZE",	PARAM_DCACHESIZE},
 	{"NUMIPTENT",	PARAM_NUMIPTENT},
 	{"AVNUMPROC",	PARAM_AVNUMPROC},
+	{"SWAPPAGES",	PARAM_SWAPPAGES},
 	{NULL, 0},
 };
 
@@ -75,10 +77,18 @@ int check_ub(ub_param *ub)
 
 #define CHECK_UB(name)							\
 if (ub->name == NULL) {							\
-	logger(-1, 0, "UB parameter " #name " not set");			\
+	logger(-1, 0, "Error: required UB parameter " #name " not set");\
 	ret = VZ_NOTENOUGHUBCPARAMS;					\
 }
 
+	if (is_vswap_config(ub))
+	{
+		CHECK_UB(physpages);
+		CHECK_UB(swappages);
+		return ret;
+	}
+	/* else
+	 * non-vswap case, require all params except swappages */
 	CHECK_UB(kmemsize)
 	CHECK_UB(lockedpages)
 	CHECK_UB(privvmpages)
@@ -128,6 +138,7 @@ inline static int is_ub_empty(ub_param *ub)
 	CHECK_UB(numfile)
 	CHECK_UB(dcachesize)
 	CHECK_UB(numiptent)
+	CHECK_UB(swappages)
 #undef CHECK_UB
 
 	return 1;
@@ -143,7 +154,7 @@ int get_ub_resid(char *name)
 	return -1;
 }
 
-const char *get_ub_name(int res_id)
+const char *get_ub_name(unsigned int res_id)
 {
 	int i;
 
@@ -155,12 +166,18 @@ const char *get_ub_name(int res_id)
 
 int set_ublimit(vps_handler *h, envid_t veid, ub_param *ub)
 {
+	int vswap = is_vswap_config(ub);
+	const unsigned long unlim_ub[2] = {LONG_MAX, LONG_MAX};
+	const unsigned long *res;
 
 #define SET_UB_LIMIT(name, id)						\
-if (ub->name != NULL) {							\
-	if (setublimit(veid, id, ub->name)) {				\
+res = ub->name;								\
+if (vswap && !res)							\
+	res = unlim_ub;							\
+if (res != NULL) {							\
+	if (setublimit(veid, id, res)) {				\
 		logger(-1, errno, "setublimit %s %lu:%lu failed",	\
-			get_ub_name(id), ub->name[0], ub->name[1]);	\
+			get_ub_name(id), res[0], res[1]);		\
 		return VZ_SETUBC_ERROR;					\
 	}								\
 }
@@ -185,6 +202,16 @@ if (ub->name != NULL) {							\
 	SET_UB_LIMIT(numfile, UB_NUMFILE)
 	SET_UB_LIMIT(dcachesize, UB_DCACHESIZE)
 	SET_UB_LIMIT(numiptent, UB_IPTENTRIES)
+	if (ub->swappages &&
+	    setublimit(veid, UB_SWAPPAGES, ub->swappages) == -1)
+	{
+		if (errno == EINVAL) {
+			logger(-1, ENOSYS, "failed to set swappages");
+		} else {
+			logger(-1, errno, "failed to set swappages");
+			return VZ_SETUBC_ERROR;
+		}
+	}
 #undef SET_UB_LIMIT
 
 	return 0;
@@ -210,13 +237,13 @@ int vps_set_ublimit(vps_handler *h, envid_t veid, ub_param *ub)
 	}
 	if ((ret = set_ublimit(h, veid, ub)))
 		return ret;
-	logger(-1, 0, "UB limits were set successfully");
+	logger(0, 0, "UB limits were set successfully");
 	return 0;
 }
 
 void free_ub_param(ub_param *ub)
 {
-#define FREE_P(x) if (ub->x != NULL) {free(ub->x); ub->x = NULL;}
+#define FREE_P(x) free(ub->x); ub->x = NULL;
 	if (ub == NULL)
 		return;
 	FREE_P(kmemsize)
@@ -240,6 +267,7 @@ void free_ub_param(ub_param *ub)
 	FREE_P(dcachesize)
 	FREE_P(numiptent)
 	FREE_P(avnumproc)
+	FREE_P(swappages)
 #undef FREE_P
 }
 
@@ -272,6 +300,7 @@ if (res_id == id) {						\
 	ADD_UB_PARAM(dcachesize, PARAM_DCACHESIZE)
 	ADD_UB_PARAM(numiptent, PARAM_NUMIPTENT)
 	ADD_UB_PARAM(avnumproc, PARAM_AVNUMPROC)
+	ADD_UB_PARAM(swappages, PARAM_SWAPPAGES)
 #undef ADD_UB_PARAM
 }
 
@@ -299,9 +328,9 @@ int add_ub_param(ub_param *ub, ub_res *res)
 void merge_ub(ub_param *dst, ub_param *src)
 {
 #define MERGE_P2(x)						\
-if ((src->x) != NULL) { 					\
-	if ((dst->x) == NULL) 					\
-		dst->x = malloc(sizeof(*(dst->x)) * 2);	 	\
+if ((src->x) != NULL) {						\
+	if ((dst->x) == NULL)					\
+		dst->x = malloc(sizeof(*(dst->x)) * 2);		\
 	dst->x[0] = src->x[0];					\
 	dst->x[1] = src->x[1];					\
 }
@@ -327,6 +356,7 @@ if ((src->x) != NULL) { 					\
 	MERGE_P2(dcachesize)
 	MERGE_P2(numiptent)
 	MERGE_P2(avnumproc)
+	MERGE_P2(swappages)
 #undef MERGE_P2
 }
 
@@ -342,7 +372,8 @@ int vps_read_ubc(envid_t veid, ub_param *ub)
 	char str[STR_SIZE];
 	char name[64];
 	const char *fmt = NULL; /* make gcc happy */
-	int ret, found, id;
+	int ret, found;
+	envid_t id;
 	unsigned long held, maxheld, barrier, limit;
 	ub_res res;
 
@@ -381,3 +412,14 @@ int vps_read_ubc(envid_t veid, ub_param *ub)
 	return !found;
 }
 
+int is_vswap_config(const ub_param *param)
+{
+	/* Dirty hack: treat INT_MAX (i.e. 32 bit LONG_MAX) as unlimited.
+	 * Required for old CTs migrated from 32 bit to 64 bit host.
+	 */
+	return  (param != NULL) &&
+		(param->physpages != NULL) &&
+		(param->physpages[1] != LONG_MAX) &&
+		(param->physpages[1] != INT_MAX);
+
+}
