@@ -34,6 +34,7 @@
 #include "config.h"
 #include "vzerror.h"
 #include "create.h"
+#include "destroy.h"
 #include "util.h"
 #include "lock.h"
 #include "vps_configure.h"
@@ -136,10 +137,14 @@ static struct option set_opt[] = {
 	{"description",	required_argument, NULL, PARAM_DESCRIPTION},
 	{"bootorder",	required_argument, NULL, PARAM_BOOTORDER},
 
+	/* New "easy" VSwap parameters */
+	{"ram",		required_argument, NULL, PARAM_RAM},
+	{"swap",	required_argument, NULL, PARAM_SWAP},
+
 	{NULL, 0, NULL, 0}
 };
 
-int parse_opt(envid_t veid, int argc, char *argv[], struct option *opt,
+static int parse_opt(envid_t veid, int argc, char *argv[], struct option *opt,
 	vps_param *param)
 {
 	int c, ret;
@@ -492,8 +497,36 @@ err_syntax:
 	return VZ_INVALID_PARAMETER_SYNTAX;
 }
 
+static int check_set_ugidlimit(unsigned long *cur, unsigned long *old,
+		int loud)
+{
+	unsigned long c, o = 0;
+
+	if (!cur)
+		return 0;
+	c = *cur;
+	if (old)
+		o = *old;
+
+	if (c != 0 && o == 0) {
+		if (loud)
+			logger(-1, 0, "Unable to turn on second-level"
+				" disk quota on a running container");
+		return 1;
+	}
+
+	if (c == 0 && o != 0) {
+		if (loud)
+			logger(-1, 0, "Unable to turn off second-level"
+				" disk quota on a running container");
+		return 1;
+	}
+
+	return 0;
+}
+
 /* Check parameters that can't be set on running CT */
-int check_set_mode(vps_handler *h, envid_t veid, int setmode, int apply,
+static int check_set_mode(vps_handler *h, envid_t veid, int setmode, int apply,
 	vps_res *new_res, vps_res *old_res)
 {
 	int found = 0;
@@ -531,20 +564,9 @@ int check_set_mode(vps_handler *h, envid_t veid, int setmode, int apply,
 					"on a running container");
 		found++;
 	}
-	/* Setting/changing quotaugidlimit */
-	if (new_res->dq.ugidlimit) {
-		if (!old_res->dq.ugidlimit) {
-			if (loud)
-				logger(-1, 0, "Unable to turn on second-level"
-					" disk quota on a running container");
-			found++;
-		} else if (*new_res->dq.ugidlimit != *old_res->dq.ugidlimit) {
-			if (loud)
-				logger(-1, 0, "Unable to change quota ugid "
-					"limit on a running container");
-			found++;
-		}
-	}
+	/* Turning quota ugid limit on/off */
+	found += check_set_ugidlimit(new_res->dq.ugidlimit,
+			old_res->dq.ugidlimit, loud);
 	/* Enabling/disabling DISK_QUOTA */
 	if ( (new_res->dq.enable) &&
 			(new_res->dq.enable != old_res->dq.enable) ) {
@@ -649,15 +671,40 @@ static int set_ve0(vps_handler *h, vps_param *g_p,
 	return 0;
 }
 
+/* If we are adding an IP, and the same IP is already in config,
+ * add it to 'del' list as well, meaning we will remove and add it again.
+ * This is mostly helpful when we do --ipadd with the same IP but
+ * different netmask (note that find_ip() is discarding netmask).
+ */
+static int fix_ip_param(vps_param *conf, vps_param *cmd)
+{
+	ip_param *ip;
+	char *found;
+	list_head_t *cur = &conf->res.net.ip;
+	list_head_t *add = &cmd->res.net.ip;
+	list_head_t *del = &cmd->del_res.net.ip;
+
+	if (list_empty(add))
+		return 0;
+
+	list_for_each(ip, add, list) {
+		found = find_ip(cur, ip->val);
+		if ((found) && (!find_ip(del, ip->val))) {
+			add_str_param(del, found);
+		}
+	}
+
+	return 0;
+}
+
 static int set(vps_handler *h, envid_t veid, vps_param *g_p, vps_param *vps_p,
 	vps_param *cmd_p)
 {
-	int ret, is_run;
+	int ret = 0, is_run;
 	dist_actions *actions = NULL;
 	char *dist_name;
 
-	ret = 0;
-
+	fix_ip_param(g_p, cmd_p);
 	if (!list_empty(&cmd_p->res.veth.dev) ||
 	    !list_empty(&cmd_p->del_res.veth.dev))
 	{
@@ -796,7 +843,6 @@ static int umount(vps_handler *h, envid_t veid, vps_param *g_p,
 static int enter(vps_handler *h, envid_t veid, const char *root,
 		int argc, char **argv)
 {
-	set_log_file(NULL);
 	if (check_var(root, "VE_ROOT is not set"))
 		return VZ_VE_ROOT_NOTSET;
 	if (!vps_is_run(h, veid)) {
@@ -807,6 +853,7 @@ static int enter(vps_handler *h, envid_t veid, const char *root,
 		/* omit "--exec" argument */
 		argc -= 1; argv += 1;
 	}
+	logger(1, 0, "Entering CT");
 	return do_enter(h, veid, root, argc, argv);
 }
 
@@ -820,30 +867,30 @@ static int exec(vps_handler *h, act_t action, envid_t veid, const char *root,
 	int i, ret;
 	int len, totallen = 0;
 
-	/* disable logging on terminal  */
-	set_log_verbose(0);
 	mode = MODE_BASH;
+	/* Unroll argv */
+	for (i = 0; i < argc; i++) {
+		len = strlen(argv[i]);
+		if (len) {
+			buf = (char*)realloc(buf, totallen + len + 2);
+			sprintf(buf + totallen, "%s ", argv[i]);
+		} else {
+			/* set empty argument */
+			len = 2;
+			buf = (char*)realloc(buf, totallen + len + 2);
+			sprintf(buf + totallen, "\'\' ");
+		}
+		totallen += len + 1;
+		buf[totallen] = 0;
+	}
 	if (action == ACTION_EXEC3) {
 		arg = argv;
 		mode = MODE_EXEC;
 	} else if (argc && strcmp(argv[0], "-")) {
-		for (i = 0; i < argc; i++) {
-			len = strlen(argv[i]);
-			if (len) {
-				buf = (char*)realloc(buf, totallen + len + 2);
-				sprintf(buf + totallen, "%s ", argv[i]);
-			} else {
-				/* set empty argument */
-				len = 2;
-				buf = (char*)realloc(buf, totallen + len + 2);
-				sprintf(buf + totallen, "\'\' ");
-			}
-			totallen += len + 1;
-			buf[totallen] = 0;
-		}
 		argv_bash[2] = buf;
 		arg = argv_bash;
 	}
+	logger(1, 0, "Executing command: %s", buf);
 	ret = vps_exec(h, veid, root, mode, arg, NULL, NULL, 0);
 
 	return ret;
@@ -1066,9 +1113,11 @@ int run_action(envid_t veid, act_t action, vps_param *g_p, vps_param *vps_p,
 					cmd_p, vps_p, &g_action);
 		} else if (cmd_p->opt.save != NO) {
 			if (list_empty(&cmd_p->res.misc.userpw)) {
+				int is_run = h != NULL && vps_is_run(h, veid);
 				logger(0, 0, "WARNING: Settings were not saved"
-				" and will be reset to original values at the"
-				" next start (use --save flag)");
+				" to config %s(use --save flag)",
+				is_run ? "and will be lost after CT restart "
+					: "");
 			}
 		}
 		break;
