@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2010, Parallels, Inc. All rights reserved.
+ *  Copyright (C) 2000-2012, Parallels, Inc. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -205,6 +205,16 @@ static void print_bootorder(struct Cveinfo *p, int index)
 				"%10lu", p->bootorder[index]);
 }
 
+static void print_cpunum(struct Cveinfo *p, int index)
+{
+	if (p->cpunum <= 0)
+		p_outbuffer += snprintf(p_outbuffer, e_buf - p_outbuffer,
+				"%5s", "-");
+	else
+		p_outbuffer += snprintf(p_outbuffer, e_buf - p_outbuffer,
+				"%5d", p->cpunum);
+}
+
 #define PRINT_UBC(name)							\
 static void print_ubc_ ## name(struct Cveinfo *p, int index)		\
 {									\
@@ -277,7 +287,7 @@ static int laverage_sort_fn(const void *val1, const void *val2)
 	const struct Ccpustat *st2 = ((const struct Cveinfo *)val2)->cpustat;
 	int res;
 
-	if ((res = check_empty_param(st1, st2)) == 2)
+	if ((res = check_empty_param(st1, st2)) != 2)
 		return res;
 	res = (st1->la[0] - st2->la[0]) * 100;
 	if (res != 0)
@@ -345,6 +355,12 @@ static int ioprio_sort_fn(const void *val1, const void *val2)
 		((const struct Cveinfo *)val2)->io.ioprio;
 }
 
+static int cpunum_sort_fn(const void *val1, const void *val2)
+{
+	return ((const struct Cveinfo *)val1)->cpunum >
+		((const struct Cveinfo *)val2)->cpunum;
+}
+
 #define SORT_STR_FN(name)						\
 static int name ## _sort_fn(const void *val1, const void *val2)		\
 {									\
@@ -413,6 +429,13 @@ SORT_DQ(diskinodes)
 SORT_UL_RES(cpulimit_sort_fn, cpu, limit, 0)
 SORT_UL_RES(cpuunits_sort_fn, cpu, limit, 1)
 
+#define UBC_FIELD(name, header) \
+{#name,      #header,      "%10s", 0, RES_UBC, print_ubc_ ## name, name ## _h_sort_fn},	\
+{#name ".m", #header ".M", "%10s", 1, RES_UBC, print_ubc_ ## name, name ## _m_sort_fn},	\
+{#name ".b", #header ".B", "%10s", 2, RES_UBC, print_ubc_ ## name, name ## _b_sort_fn},	\
+{#name ".l", #header ".L", "%10s", 3, RES_UBC, print_ubc_ ## name, name ## _l_sort_fn},	\
+{#name ".f", #header ".F", "%10s", 4, RES_UBC, print_ubc_ ## name, name ## _f_sort_fn}
+
 static struct Cfield field_names[] =
 {
 /* ctid should have index 0 */
@@ -429,130 +452,27 @@ static struct Cfield field_names[] =
 {"ip", "IP_ADDR", "%-15s", 0, RES_IP, print_ip, ip_sort_fn},
 {"status", "STATUS", "%-9s", 0, RES_NONE, print_status, status_sort_fn},
 /*	UBC	*/
-{"kmemsize", "KMEMSIZE", "%10s", 0, RES_UBC, print_ubc_kmemsize, kmemsize_h_sort_fn},
-{"kmemsize.m", "KMEMSIZE.M", "%10s", 1, RES_UBC, print_ubc_kmemsize, kmemsize_m_sort_fn},
-{"kmemsize.b", "KMEMSIZE.B", "%10s", 2, RES_UBC, print_ubc_kmemsize, kmemsize_b_sort_fn},
-{"kmemsize.l", "KMEMSIZE.L", "%10s", 3, RES_UBC, print_ubc_kmemsize, kmemsize_l_sort_fn},
-{"kmemsize.f", "KMEMSIZE.F", "%10s", 4, RES_UBC, print_ubc_kmemsize, kmemsize_f_sort_fn},
-
-{"lockedpages", "LOCKEDP", "%10s", 0, RES_UBC, print_ubc_lockedpages, lockedpages_h_sort_fn},
-{"lockedpages.m", "LOCKEDP.M", "%10s", 1, RES_UBC, print_ubc_lockedpages, lockedpages_m_sort_fn},
-{"lockedpages.b", "LOCKEDP.B", "%10s", 2, RES_UBC, print_ubc_lockedpages, lockedpages_b_sort_fn},
-{"lockedpages.l", "LOCKEDP.L", "%10s", 3, RES_UBC, print_ubc_lockedpages, lockedpages_l_sort_fn},
-{"lockedpages.f", "LOCKEDP.F", "%10s", 4, RES_UBC, print_ubc_lockedpages, lockedpages_f_sort_fn},
-
-{"privvmpages", "PRIVVMP", "%10s", 0, RES_UBC, print_ubc_privvmpages, privvmpages_h_sort_fn},
-{"privvmpages.m", "PRIVVMP.M", "%10s", 1, RES_UBC, print_ubc_privvmpages, privvmpages_m_sort_fn},
-{"privvmpages.b", "PRIVVMP.B", "%10s", 2, RES_UBC, print_ubc_privvmpages, privvmpages_b_sort_fn},
-{"privvmpages.l", "PRIVVMP.L", "%10s", 3, RES_UBC, print_ubc_privvmpages, privvmpages_l_sort_fn},
-{"privvmpages.f", "PRIVVMP.F", "%10s", 4, RES_UBC, print_ubc_privvmpages, privvmpages_f_sort_fn},
-
-{"shmpages", "SHMP", "%10s", 0, RES_UBC, print_ubc_shmpages, shmpages_h_sort_fn},
-{"shmpages.m", "SHMP.M", "%10s", 1, RES_UBC, print_ubc_shmpages, shmpages_m_sort_fn},
-{"shmpages.b", "SHMP.B", "%10s", 2, RES_UBC, print_ubc_shmpages, shmpages_b_sort_fn},
-{"shmpages.l", "SHMP.L", "%10s", 3, RES_UBC, print_ubc_shmpages, shmpages_l_sort_fn},
-{"shmpages.f", "SHMP.F", "%10s", 4, RES_UBC, print_ubc_shmpages, shmpages_f_sort_fn},
-{"numproc", "NPROC", "%10s", 0, RES_UBC, print_ubc_numproc, numproc_h_sort_fn},
-{"numproc.m", "NPROC.M", "%10s", 1, RES_UBC, print_ubc_numproc, numproc_m_sort_fn},
-{"numproc.b", "NPROC.B", "%10s", 2, RES_UBC, print_ubc_numproc, numproc_b_sort_fn},
-{"numproc.l", "NPROC.L", "%10s", 3, RES_UBC, print_ubc_numproc, numproc_l_sort_fn},
-{"numproc.f", "NPROC.F", "%10s", 4, RES_UBC, print_ubc_numproc, numproc_f_sort_fn},
-
-{"physpages", "PHYSP", "%10s", 0, RES_UBC, print_ubc_physpages, physpages_h_sort_fn},
-{"physpages.m", "PHYSP.M", "%10s", 1, RES_UBC, print_ubc_physpages, physpages_m_sort_fn},
-{"physpages.b", "PHYSP.B", "%10s", 2, RES_UBC, print_ubc_physpages, physpages_b_sort_fn},
-{"physpages.l", "PHYSP.L", "%10s", 3, RES_UBC, print_ubc_physpages, physpages_l_sort_fn},
-{"physpages.f", "PHYSP.F", "%10s", 4, RES_UBC, print_ubc_physpages, physpages_f_sort_fn},
-
-{"vmguarpages", "VMGUARP", "%10s", 0, RES_UBC, print_ubc_vmguarpages, vmguarpages_h_sort_fn},
-{"vmguarpages.m", "VMGUARP.M", "%10s", 1, RES_UBC,print_ubc_vmguarpages, vmguarpages_m_sort_fn},
-{"vmguarpages.b", "VMGUARP.B", "%10s", 2, RES_UBC,print_ubc_vmguarpages, vmguarpages_b_sort_fn},
-{"vmguarpages.l", "VMGUARP.L", "%10s", 3, RES_UBC,print_ubc_vmguarpages, vmguarpages_l_sort_fn},
-{"vmguarpages.f", "VMGUARP.F", "%10s", 4, RES_UBC,print_ubc_vmguarpages, vmguarpages_f_sort_fn},
-
-{"oomguarpages", "OOMGUARP", "%10s", 0, RES_UBC,print_ubc_oomguarpages, oomguarpages_h_sort_fn},
-{"oomguarpages.m", "OOMGUARP.M", "%10s", 1, RES_UBC,print_ubc_oomguarpages, oomguarpages_m_sort_fn},
-{"oomguarpages.b", "OOMGUARP.B", "%10s", 2, RES_UBC,print_ubc_oomguarpages, oomguarpages_b_sort_fn},
-{"oomguarpages.l", "OOMGUARP.L", "%10s", 3, RES_UBC,print_ubc_oomguarpages, oomguarpages_l_sort_fn},
-{"oomguarpages.f", "OOMGUARP.F", "%10s", 4, RES_UBC,print_ubc_oomguarpages, oomguarpages_f_sort_fn},
-
-{"numtcpsock", "NTCPSOCK", "%10s", 0, RES_UBC,print_ubc_numtcpsock, numtcpsock_h_sort_fn},
-{"numtcpsock.m", "NTCPSOCK.M", "%10s", 1, RES_UBC,print_ubc_numtcpsock, numtcpsock_m_sort_fn},
-{"numtcpsock.b", "NTCPSOCK.B", "%10s", 2, RES_UBC,print_ubc_numtcpsock, numtcpsock_b_sort_fn},
-{"numtcpsock.l", "NTCPSOCK.L", "%10s", 3, RES_UBC,print_ubc_numtcpsock, numtcpsock_l_sort_fn},
-{"numtcpsock.f", "NTCPSOCK.F", "%10s", 4, RES_UBC,print_ubc_numtcpsock, numtcpsock_f_sort_fn},
-
-{"numflock", "NFLOCK", "%10s", 0, RES_UBC,print_ubc_numflock, numflock_h_sort_fn},
-{"numflock.m", "NFLOCK.M", "%10s", 1, RES_UBC,print_ubc_numflock, numflock_m_sort_fn},
-{"numflock.b", "NFLOCK.B", "%10s", 2, RES_UBC,print_ubc_numflock, numflock_b_sort_fn},
-{"numflock.l", "NFLOCK.L", "%10s", 3, RES_UBC,print_ubc_numflock, numflock_l_sort_fn},
-{"numflock.f", "NFLOCK.F", "%10s", 4, RES_UBC,print_ubc_numflock, numflock_f_sort_fn},
-
-{"numpty", "NPTY", "%10s", 0, RES_UBC,print_ubc_numpty, numpty_h_sort_fn},
-{"numpty.m", "NPTY.M", "%10s", 1, RES_UBC,print_ubc_numpty, numpty_m_sort_fn},
-{"numpty.b", "NPTY.B", "%10s", 2, RES_UBC,print_ubc_numpty, numpty_b_sort_fn},
-{"numpty.l", "NPTY.L", "%10s", 3, RES_UBC,print_ubc_numpty, numpty_l_sort_fn},
-{"numpty.f", "NPTY.F", "%10s", 4, RES_UBC,print_ubc_numpty, numpty_f_sort_fn},
-
-{"numsiginfo", "NSIGINFO", "%10s", 0, RES_UBC,print_ubc_numsiginfo, numsiginfo_h_sort_fn},
-{"numsiginfo.m", "NSIGINFO.M", "%10s", 1, RES_UBC,print_ubc_numsiginfo, numsiginfo_m_sort_fn},
-{"numsiginfo.b", "NSIGINFO.B", "%10s", 2, RES_UBC,print_ubc_numsiginfo, numsiginfo_b_sort_fn},
-{"numsiginfo.l", "NSIGINFO.L", "%10s", 3, RES_UBC,print_ubc_numsiginfo, numsiginfo_l_sort_fn},
-{"numsiginfo.f", "NSIGINFO.F", "%10s", 4, RES_UBC,print_ubc_numsiginfo, numsiginfo_f_sort_fn},
-
-{"tcpsndbuf", "TCPSNDB", "%10s", 0, RES_UBC,print_ubc_tcpsndbuf, tcpsndbuf_h_sort_fn},
-{"tcpsndbuf.m", "TCPSNDB.M", "%10s", 1, RES_UBC,print_ubc_tcpsndbuf, tcpsndbuf_m_sort_fn},
-{"tcpsndbuf.b", "TCPSNDB.B", "%10s", 2, RES_UBC,print_ubc_tcpsndbuf, tcpsndbuf_b_sort_fn},
-{"tcpsndbuf.l", "TCPSNDB.L", "%10s", 3, RES_UBC,print_ubc_tcpsndbuf, tcpsndbuf_l_sort_fn},
-{"tcpsndbuf.f", "TCPSNDB.F", "%10s", 4, RES_UBC,print_ubc_tcpsndbuf, tcpsndbuf_f_sort_fn},
-
-{"tcprcvbuf", "TCPRCVB", "%10s", 0, RES_UBC,print_ubc_tcprcvbuf, tcprcvbuf_h_sort_fn},
-{"tcprcvbuf.m", "TCPRCVB.M", "%10s", 1, RES_UBC,print_ubc_tcprcvbuf, tcprcvbuf_m_sort_fn},
-{"tcprcvbuf.b", "TCPRCVB.B", "%10s", 2, RES_UBC,print_ubc_tcprcvbuf, tcprcvbuf_b_sort_fn},
-{"tcprcvbuf.l", "TCPRCVB.L", "%10s", 3, RES_UBC,print_ubc_tcprcvbuf, tcprcvbuf_l_sort_fn},
-{"tcprcvbuf.f", "TCPRCVB.F", "%10s", 4, RES_UBC,print_ubc_tcprcvbuf, tcprcvbuf_f_sort_fn},
-
-{"othersockbuf", "OTHSOCKB", "%10s", 0, RES_UBC,print_ubc_othersockbuf, othersockbuf_h_sort_fn},
-{"othersockbuf.m", "OTHSOCKB.M", "%10s", 1, RES_UBC,print_ubc_othersockbuf, othersockbuf_m_sort_fn},
-{"othersockbuf.b", "OTHSOCKB.B", "%10s", 2, RES_UBC,print_ubc_othersockbuf, othersockbuf_b_sort_fn},
-{"othersockbuf.l", "OTHSOCKB.L", "%10s", 3, RES_UBC,print_ubc_othersockbuf, othersockbuf_l_sort_fn},
-{"othersockbuf.f", "OTHSOCKB.F", "%10s", 4, RES_UBC,print_ubc_othersockbuf, othersockbuf_f_sort_fn},
-
-{"dgramrcvbuf", "DGRAMRB", "%10s", 0, RES_UBC,print_ubc_dgramrcvbuf, dgramrcvbuf_h_sort_fn},
-{"dgramrcvbuf.m", "DGRAMRB.M", "%10s", 1, RES_UBC,print_ubc_dgramrcvbuf, dgramrcvbuf_m_sort_fn},
-{"dgramrcvbuf.b", "DGRAMRB.B", "%10s", 2, RES_UBC,print_ubc_dgramrcvbuf, dgramrcvbuf_b_sort_fn},
-{"dgramrcvbuf.l", "DGRAMRB.L", "%10s", 3, RES_UBC,print_ubc_dgramrcvbuf, dgramrcvbuf_l_sort_fn},
-{"dgramrcvbuf.f", "DGRAMRB.F", "%10s", 4, RES_UBC,print_ubc_dgramrcvbuf, dgramrcvbuf_f_sort_fn},
-
-{"numothersock", "NOTHSOCK", "%10s", 0, RES_UBC,print_ubc_numothersock, numothersock_h_sort_fn},
-{"numothersock.m", "NOTHSOCK.M", "%10s", 1, RES_UBC,print_ubc_numothersock, numothersock_m_sort_fn},
-{"numothersock.b", "NOTHSOCK.B", "%10s", 2, RES_UBC,print_ubc_numothersock, numothersock_b_sort_fn},
-{"numothersock.l", "NOTHSOCK.L", "%10s", 3, RES_UBC,print_ubc_numothersock, numothersock_l_sort_fn},
-{"numothersock.f", "NOTHSOCK.F", "%10s", 4, RES_UBC,print_ubc_numothersock, numothersock_f_sort_fn},
-
-{"dcachesize", "DCACHESZ", "%10s", 0, RES_UBC,print_ubc_dcachesize, dcachesize_h_sort_fn},
-{"dcachesize.m", "DCACHESZ.M", "%10s", 1, RES_UBC,print_ubc_dcachesize, dcachesize_m_sort_fn},
-{"dcachesize.b", "DCACHESZ.B", "%10s", 2, RES_UBC,print_ubc_dcachesize, dcachesize_b_sort_fn},
-{"dcachesize.l", "DCACHESZ.L", "%10s", 3, RES_UBC,print_ubc_dcachesize, dcachesize_l_sort_fn},
-{"dcachesize.f", "DCACHESZ.F", "%10s", 4, RES_UBC,print_ubc_dcachesize, dcachesize_f_sort_fn},
-
-{"numfile", "NFILE", "%10s", 0, RES_UBC,print_ubc_numfile, numfile_h_sort_fn},
-{"numfile.m", "NFILE.M", "%10s", 1, RES_UBC,print_ubc_numfile, numfile_m_sort_fn},
-{"numfile.b", "NFILE.B", "%10s", 2, RES_UBC,print_ubc_numfile, numfile_b_sort_fn},
-{"numfile.l", "NFILE.L", "%10s", 3, RES_UBC,print_ubc_numfile, numfile_l_sort_fn},
-{"numfile.f", "NFILE.F", "%10s", 4, RES_UBC,print_ubc_numfile, numfile_f_sort_fn},
-
-{"numiptent", "NIPTENT", "%10s", 0, RES_UBC,print_ubc_numiptent, numiptent_h_sort_fn},
-{"numiptent.m", "NIPTENT.M", "%10s", 1, RES_UBC,print_ubc_numiptent, numiptent_m_sort_fn},
-{"numiptent.b", "NIPTENT.B", "%10s", 2, RES_UBC,print_ubc_numiptent, numiptent_b_sort_fn},
-{"numiptent.l", "NIPTENT.L", "%10s", 3, RES_UBC,print_ubc_numiptent, numiptent_l_sort_fn},
-{"numiptent.f", "NIPTENT.F", "%10s", 4, RES_UBC,print_ubc_numiptent, numiptent_f_sort_fn},
-
-{"swappages", "SWAPP", "%10s", 0, RES_UBC,print_ubc_swappages, swappages_h_sort_fn},
-{"swappages.m", "SWAPP.M", "%10s", 1, RES_UBC,print_ubc_swappages, swappages_m_sort_fn},
-{"swappages.b", "SWAPP.B", "%10s", 2, RES_UBC,print_ubc_swappages, swappages_b_sort_fn},
-{"swappages.l", "SWAPP.L", "%10s", 3, RES_UBC,print_ubc_swappages, swappages_l_sort_fn},
-{"swappages.f", "SWAPP.F", "%10s", 4, RES_UBC,print_ubc_swappages, swappages_f_sort_fn},
+UBC_FIELD(kmemsize, KMEMSIZE),
+UBC_FIELD(lockedpages, LOCKEDP),
+UBC_FIELD(privvmpages, PRIVVMP),
+UBC_FIELD(shmpages, SHMP),
+UBC_FIELD(numproc, NPROC),
+UBC_FIELD(physpages, PHYSP),
+UBC_FIELD(vmguarpages, VMGUARP),
+UBC_FIELD(oomguarpages, OOMGUARP),
+UBC_FIELD(numtcpsock, NTCPSOCK),
+UBC_FIELD(numflock, NFLOCK),
+UBC_FIELD(numpty, NPTY),
+UBC_FIELD(numsiginfo, NSIGINFO),
+UBC_FIELD(tcpsndbuf, TCPSNDB),
+UBC_FIELD(tcprcvbuf, TCPRCVB),
+UBC_FIELD(othersockbuf, OTHSOCKB),
+UBC_FIELD(dgramrcvbuf, DGRAMRB),
+UBC_FIELD(numothersock, NOTHSOCK),
+UBC_FIELD(dcachesize, DCACHESZ),
+UBC_FIELD(numfile, NFILE),
+UBC_FIELD(numiptent, NIPTENT),
+UBC_FIELD(swappages, SWAPP),
 
 {"diskspace", "DSPACE", "%10s", 0, RES_QUOTA, print_diskspace, diskspace_u_sort_fn},
 {"diskspace.s", "DSPACE.S", "%10s", 1, RES_QUOTA, print_diskspace, diskspace_s_sort_fn},
@@ -563,12 +483,13 @@ static struct Cfield field_names[] =
 {"diskinodes.h", "DINODES.H", "%10s", 2, RES_QUOTA, print_diskinodes, diskinodes_h_sort_fn},
 
 {"laverage", "LAVERAGE", "%14s", 0, RES_CPUSTAT, print_laverage, laverage_sort_fn},
-{"uptime", "UPTIME", "%15s", 1, RES_CPUSTAT, print_uptime, uptime_sort_fn},
+{"uptime", "UPTIME", "%15s", 0, RES_CPUSTAT, print_uptime, uptime_sort_fn},
 
 {"cpulimit", "CPULIM", "%7s", 0, RES_CPU, print_cpulimit, cpulimit_sort_fn},
 {"cpuunits", "CPUUNI", "%7s", 1, RES_CPU, print_cpulimit, cpuunits_sort_fn},
+{"cpus", "CPUS", "%5s", 0, RES_CPUNUM, print_cpunum, cpunum_sort_fn},
 
-{"ioprio", "IOP", "%3s", 1, RES_NONE, print_ioprio, ioprio_sort_fn},
+{"ioprio", "IOP", "%3s", 0, RES_NONE, print_ioprio, ioprio_sort_fn},
 
 {"onboot", "ONBOOT", "%6s", 0, RES_NONE, print_onboot, none_sort_fn},
 {"bootorder", "BOOTORDER", "%10s", 0, RES_NONE,
@@ -579,7 +500,7 @@ static void *x_malloc(int size)
 {
 	void *p;
 	if ((p = malloc(size)) == NULL) {
-		printf("Error: unable to allocate %d bytes\n", size);
+		fprintf(stderr, "Error: unable to allocate %d bytes\n", size);
 		exit(1);
 	}
 	return p;
@@ -590,7 +511,7 @@ static void *x_realloc(void *ptr, int size)
 	void *tmp;
 
 	if ((tmp = realloc(ptr, size)) == NULL) {
-		printf("Error: unable to allocate %d bytes\n", size);
+		fprintf(stderr, "Error: unable to allocate %d bytes\n", size);
 		exit(1);
 	}
 	return tmp;
@@ -598,10 +519,10 @@ static void *x_realloc(void *ptr, int size)
 
 static void usage()
 {
-	fprintf(stderr, "\n"
-"Usage:	vzlist	[-a|-S] [-n] [-H] [-o field[,field...]] [-s [-]field]\n"
-"		[-h pattern] [-N pattern] [-d pattern] [CTID [CTID ...]|-1]\n"
-"	vzlist -L\n"
+	printf(
+"Usage:	vzlist [-a | -S] [-n] [-H] [-o field[,field...] | -1] [-s [-]field]\n"
+"	       [-h pattern] [-N pattern] [-d pattern] [CTID [CTID ...]]\n"
+"	vzlist -L | --list\n"
 "\n"
 "Options:\n"
 "	-a, --all		list all containers\n"
@@ -609,6 +530,7 @@ static void usage()
 "	-n, --name		display containers' names\n"
 "	-H, --no-header		suppress columns header\n"
 "	-o, --output		output only specified fields\n"
+"	-1			synonym for -H -octid\n"
 "	-s, --sort		sort by the specified field\n"
 "				('-field' to reverse sort order)\n"
 "	-h, --hostname		filter CTs by hostname pattern\n"
@@ -811,17 +733,6 @@ static void update_cpu(int veid, unsigned long limit, unsigned long units)
 	return;
 }
 
-static void update_cpustat(int veid, struct Ccpustat *st)
-{
-	struct Cveinfo *tmp;
-
-	if ((tmp = find_ve(veid)) == NULL)
-		return;
-	tmp->cpustat = x_malloc(sizeof(*st));
-	memcpy(tmp->cpustat, st, sizeof(*st));
-	return;
-}
-
 #define MERGE_QUOTA(name, quota, dq)				\
 do {								\
 	if (dq.name != NULL) {					\
@@ -911,6 +822,8 @@ do {								\
 		*ve->bootorder = *res->misc.bootorder;
 	}
 	ve->io.ioprio = res->io.ioprio;
+	if (res->cpu.vcpus != NULL)
+		ve->cpunum = *res->cpu.vcpus;
 }
 
 static int read_ves_param()
@@ -981,10 +894,14 @@ static int get_ub()
 	char *s;
 	struct Cveinfo ve;
 
-	if ((fp = fopen(PROCUBC, "r")) == NULL) {
-		fprintf(stderr, "Unable to open %s\n", PROCUBC);
-		return 1;
+	if ((fp = fopen(PROC_BC_RES, "r")) == NULL) {
+		if ((fp = fopen(PROCUBC, "r")) == NULL) {
+			fprintf(stderr, "Unable to open %s: %s\n",
+					PROCUBC, strerror(errno));
+			return 1;
+		}
 	}
+
 	veid = 0;
 	memset(&ve, 0, sizeof(struct Cveinfo));
 	while (!feof(fp)) {
@@ -1086,7 +1003,7 @@ static char *invert_ip(char *ips)
 		strncpy(ip_str, p, len);
 		ip_str[len] = 0;
 		if ((family = get_netaddr(ip_str, ip)) == -1)
-			continue;;
+			continue;
 		if ((inet_ntop(family, ip, ip_str, sizeof(ip_str) - 1)) == NULL)
 			continue;
 		rc = sprintf(tp, "%s ", ip_str);
@@ -1106,7 +1023,8 @@ static int get_run_ve_proc(int update)
 	int res, veid, classid, nproc;
 
 	if ((fp = fopen(PROCVEINFO, "r")) == NULL) {
-		fprintf(stderr, "Unable to open %s\n", PROCVEINFO);
+		fprintf(stderr, "Unable to open %s: %s\n",
+				PROCVEINFO, strerror(errno));
 		return 1;
 	}
 	memset(&ve, 0, sizeof(struct Cveinfo));
@@ -1284,13 +1202,13 @@ static long get_clk_tck()
 	return __clk_tck;
 }
 
-static int get_ve_cpustat(int veid)
+static int get_ve_cpustat(struct Cveinfo *ve)
 {
 	struct vz_cpu_stat stat;
 	struct vzctl_cpustatctl statctl;
 	struct Ccpustat st;
 
-	statctl.veid = veid;
+	statctl.veid = ve->veid;
 	statctl.cpustat = &stat;
 	if (ioctl(vzctlfd, VZCTL_GET_CPU_STAT, &statctl) != 0)
 		return 1;
@@ -1300,7 +1218,8 @@ static int get_ve_cpustat(int veid)
 
 	st.uptime = (float) stat.uptime_jif / get_clk_tck();
 
-	update_cpustat(veid, &st);
+	ve->cpustat = x_malloc(sizeof(st));
+	memcpy(ve->cpustat, &st, sizeof(st));
 	return 0;
 }
 
@@ -1313,7 +1232,7 @@ static int get_ves_cpustat()
 	for (i = 0; i < n_veinfo; i++) {
 		if (veinfo[i].hide)
 			continue;
-		get_ve_cpustat(veinfo[i].veid);
+		get_ve_cpustat(&veinfo[i]);
 	}
 	close(vzctlfd);
 	return 0;
@@ -1344,6 +1263,42 @@ static int get_mounted_status()
 	return 0;
 }
 
+static int get_ve_cpunum(struct Cveinfo *ve) {
+	char path[] = "/proc/vz/fairsched/2147483647/cpu.nr_cpus";
+	int veid = ve->veid;
+	FILE *fp;
+	int ret = -1;
+
+	snprintf(path, sizeof(path),
+			"/proc/vz/fairsched/%d/cpu.nr_cpus", veid);
+	fp = fopen(path, "r");
+	if (fp == NULL) {
+		fprintf(stderr, "Unable to open %s: %s\n",
+				path, strerror(errno));
+		return -1;
+	}
+	if (fscanf(fp, "%d", &ve->cpunum) != 1)
+		goto out;
+
+	ret = 0;
+out:
+	fclose(fp);
+
+	return ret;
+}
+
+static int get_ves_cpunum()
+{
+	int i;
+
+	for (i = 0; i < n_veinfo; i++) {
+		if ((veinfo[i].hide) || (veinfo[i].status != VE_RUNNING))
+			continue;
+		get_ve_cpunum(&veinfo[i]);
+	}
+	return 0;
+}
+
 static int get_ves_cpu()
 {
 	unsigned long tmp;
@@ -1352,7 +1307,8 @@ static int get_ves_cpu()
 	char buf[128];
 
 	if ((fp = fopen(PROCFSHED, "r")) == NULL) {
-		fprintf(stderr, "Unable to open %s\n", PROCFSHED);
+		fprintf(stderr, "Unable to open %s: %s\n",
+				PROCFSHED, strerror(errno));
 		return 1;
 	}
 	veid = 0;
@@ -1489,6 +1445,9 @@ static int collect()
 	if (check_param(RES_CPU))
 		if (!only_stopped_ve && (ret = get_ves_cpu()))
 			return ret;
+	if (check_param(RES_CPUNUM))
+		if (!only_stopped_ve && (ret = get_ves_cpunum()))
+			return ret;
 	read_ves_param();
 	get_mounted_status();
 	if (host_pattern != NULL)
@@ -1601,7 +1560,8 @@ int main(int argc, char **argv)
 				sort_rev = 1;
 			}
 			if ((g_sort_field = search_field(p)) < 0) {
-				printf("Invalid sort field name: %s\n", optarg);
+				fprintf(stderr, "Invalid sort field name: "
+						"%s\n", optarg);
 				return 1;
 			}
 			break;
@@ -1617,6 +1577,7 @@ int main(int argc, char **argv)
 			name_pattern = strdup(optarg);
 			break;
 		default		:
+			/* "Unknown option" error msg is printed by getopt */
 			usage();
 			return 1;
 		}
